import _JSXStyle from "styled-jsx/style";
import React from "react";
import { Button } from "@orbit-ui/react-button";
import { CalendarIcon, CloseIcon, getIconSizeForControl } from "@orbit-ui/react-icons";
import { DEFAULT_SIZE, SIZES } from "./sizes";
import { KEYS, LARGE, MEDIUM, SMALL, isNullOrEmpty, mergeClasses, withHandlerProxy } from "@orbit-ui/react-components-shared";
import { PureComponent, createRef } from "react";
import { ResizeObserver } from "./resize-observer";
import { bool, func, oneOf, string } from "prop-types";
import { isNil } from "lodash";
import cx from "classnames";
const SIZES_TO_HEIGHT = {
  [SMALL]: "h7",
  [MEDIUM]: "h8",
  [LARGE]: "h9"
};
const SIZES_TO_FONT_SIZE = {
  [SMALL]: "f6",
  [MEDIUM]: "f6",
  [LARGE]: "f5"
};
export class DatePickerTextboxInput extends PureComponent {
  constructor(...args) {
    super(...args);
    this._containerRef = createRef();
    this._clearButtonRef = createRef();
    this._containerResizeObserver = null;

    this.handleContainerSizeChange = entries => {
      const {
        onSizeChange
      } = this.props;

      if (!isNil(onSizeChange)) {
        // The native chrome implementation doesn't currently support the "border-box" specs. Therefore, we rely on "getBoundingClientRect"
        // for the value when a size changed is detected.
        // Specs available here: https://drafts.csswg.org/resize-observer-1/
        const dimensions = entries[0].target.getBoundingClientRect();
        onSizeChange({
          width: dimensions.width,
          height: dimensions.height
        });
      }
    };

    this.handleClick = event => {
      const {
        onClick,
        onOpen,
        onClose,
        allowClear,
        disabled,
        open
      } = this.props;
      let canPropagate = true;

      if (allowClear) {
        canPropagate = !this._clearButtonRef.current.contains(event.target);
      }

      if (canPropagate) {
        if (!isNil(onClick)) {
          onClick(event, this.props);
        }

        if (!disabled) {
          if (!open) {
            onOpen(event, this.props);
          } else {
            onClose(event, this.props);
          }
        }
      }
    };

    this.handleKeyDown = event => {
      const {
        onKeyDown,
        onOpen,
        onClear,
        disabled,
        open
      } = this.props;

      if (!disabled) {
        const key = event.keyCode;

        if (key === KEYS.space || key === KEYS.enter) {
          if (key === KEYS.space) {
            event.preventDefault();
          }

          if (!open) {
            onOpen(event, this.props);
          }
        } else if (key === KEYS.esc) {
          if (!open) {
            onClear(event, this.props);
          }
        }
      }

      if (!isNil(onKeyDown)) {
        onKeyDown(event, this.props);
      }
    };

    this.handleFocus = withHandlerProxy(this, "onFocus");
    this.handleBlur = withHandlerProxy(this, "onBlur");
    this.handleClearButtonClick = withHandlerProxy(this, "onClear");
  }

  componentDidMount() {
    this._containerResizeObserver = new ResizeObserver(this.handleContainerSizeChange);

    this._containerResizeObserver.observe(this._containerRef.current);
  }

  componentWillUnmount() {
    this._containerResizeObserver.disconnect();
  }

  isPlaceholder() {
    const {
      value
    } = this.props;
    return isNullOrEmpty(value);
  }

  getCssClasses() {
    const {
      disabled,
      open,
      size,
      className
    } = this.props;
    return mergeClasses("input pv3 ph2 ba outline-0 f6 br2 flex items-center", open ? "b--marine-600 marine-600" : "b--cloud-200 marine-200", !this.isPlaceholder() && "marine-600", !disabled ? "hover-b--marine-600 hover-marine-600 pointer" : "bg-cloud-100 cloud-400 crsr-not-allowed", SIZES_TO_HEIGHT[size], SIZES_TO_FONT_SIZE[size], className);
  }

  renderIcon() {
    const {
      size,
      disabled
    } = this.props;
    return React.createElement(CalendarIcon, {
      size: getIconSizeForControl(size),
      className: !disabled ? "fill-marine-700" : "fill-cloud-500"
    });
  }

  renderClearButton() {
    const {
      allowClear,
      disabled,
      open
    } = this.props;

    if (!allowClear) {
      return null;
    }

    return React.createElement("div", {
      className: cx("clear-btn-container", {
        dn: this.isPlaceholder() || disabled || open
      })
    }, React.createElement(Button, {
      circular: true,
      ghost: true,
      secondary: true,
      icon: React.createElement(CloseIcon, null),
      size: "tiny",
      onClick: this.handleClearButtonClick,
      ref: this._clearButtonRef
    }));
  }

  render() {
    const {
      value,
      placeholder,
      disabled
    } = this.props;
    return React.createElement("div", {
      onClick: this.handleClick,
      onKeyDown: this.handleKeyDown,
      onFocus: this.handleFocus,
      onBlur: this.handlerBlur,
      tabIndex: disabled ? "-1" : "0",
      autoComplete: "off",
      disabled: disabled,
      ref: this._containerRef,
      className: "jsx-3833502881" + " " + (this.getCssClasses() || "")
    }, this.renderIcon(), React.createElement("span", {
      className: "jsx-3833502881" + " " + "flex-grow-1 ml2"
    }, this.isPlaceholder() ? placeholder : value), this.renderClearButton(), React.createElement(_JSXStyle, {
      id: "3833502881"
    }, [".input.jsx-3833502881{min-width:300px;}", ".input.jsx-3833502881:not(\"disabled\"){cursor:text;}", ".input.jsx-3833502881:focus{border:1px solid var(--marine-700);color:var(--marine-700);}"]));
  } // This method is part of the component external API.


  focus() {
    if (!isNil(this._containerRef.current)) {
      this._containerRef.current.focus();
    }
  }

}
DatePickerTextboxInput.displayName = "DatePickerTextboxInput";
DatePickerTextboxInput.propTypes = {
  value: string.isRequired,
  onOpen: func,
  onClose: func,
  onSizeChange: func,
  onClick: func,
  onKeyDown: func,
  // eslint-disable-next-line react/no-unused-prop-types
  onFocus: func,
  // eslint-disable-next-line react/no-unused-prop-types
  onBlur: func,
  // eslint-disable-next-line react/no-unused-prop-types
  onClear: func,
  allowClear: bool,
  placeholder: string,
  disabled: bool,
  open: bool,
  size: oneOf(SIZES),
  className: string
};
DatePickerTextboxInput.defaultProps = {
  allowClear: true,
  placeholder: "Pick a date",
  disabled: false,
  size: DEFAULT_SIZE
};